#mp3/admin/mp3-add-2.tcl
ad_page_contract {
    Read in the file specified by the form, store it in the file system,
    parse out the relevant data and store it in the database.
} {
    {upload_file ""}
    mp3_id
} -properties {
    sql:onevalue
    page_title:onevalue
    context:onevalue
}

set title "Add an MP3"
set context [list $title]
set page_title "MP3 Added"

#requires admin permission as under /admin
set package_id [ad_conn package_id]

set exception_text ""
set exception_count 0

if { ![exists_and_not_null upload_file] } {
    append exception_text "<li>You need to upload a file\n"
    incr exception_count
}

if { $exception_count > 0 } {
    ad_return_complaint $exception_count $exception_text
    return 0
}

set creation_ip [ad_conn "peeraddr"]
set creation_user [ad_conn "user_id"]

set max_n_bytes [ad_parameter MaxNumberOfBytes fs]

set tmp_filename [ns_queryget upload_file.tmpfile]
#set version_content [read [open $tmp_filename] $max_n_bytes]

set file_extension [string tolower [file extension $upload_file]]
# remove the first . from the file extension
regsub "\." $file_extension "" file_extension

set guessed_file_type [ns_guesstype $upload_file]

set n_bytes [file size $tmp_filename]

# strip off the C:\directories... crud and just get the file name
if {![regexp {([^//\\]+)$} $upload_file match client_filename]} {
    # couldn't find a match
    set client_filename $upload_file
}

if [db_0or1row check_file_name "
        select deleted_p 
        from   mp3_mp3s
        where  file_path = :client_filename"] {
    if { $deleted_p == "f" } {
	ad_return_complaint 1 "That file seems to already exist in the database"
	return
    } else {
	set update_entry_p 1
    }
} else {
    set update_entry_p 0
}

set target_filename "[ad_parameter MP3FileLocation]/$client_filename"

file copy -force -- $tmp_filename $target_filename

#regsub www [acs_root_dir] "bin/mp3_file_scan" script_path

set mp3_info_list [exec "[acs_root_dir]/packages/mp3-jukebox/bin/mp3_file_scan" $target_filename]
# Workaround due to plpgsql (as of PG 7.1.3) being unable to handle
# default values in functions):
# The number of items in "mp3_info_list" may vary so we initialize
# all the possible vars to "nullvalue" in advance to know they're always
# being set.

set nullvalue [db_null]

set mp3_title $nullvalue
set artist $nullvalue
set bitrate $nullvalue
set album $nullvalue
set tracknum $nullvalue
set genre $nullvalue
set year $nullvalue
set layer $nullvalue
set copyright_p $nullvalue
set version $nullvalue
set mode_num $nullvalue
set stereo_p $nullvalue
set frequency $nullvalue
set minutes $nullvalue
set seconds $nullvalue
set comments $nullvalue
set vbr_p $nullvalue
set state $nullvalue

foreach info_item $mp3_info_list {
    set info_key [string tolower [lindex $info_item 0]]
    set info_val [lindex $info_item 1]
    switch -exact $info_key {
	title {
	    set mp3_title [db_nullify_empty_string $info_val]
	    # title is included in the standard sql, so no need to add to info_pairs
	}
	mm {
	    set minutes [db_nullify_empty_string $info_val]
	    set info_pairs(minutes) ":minutes"
	}
	ss {
	    set seconds [db_nullify_empty_string $info_val]
	    set info_pairs(seconds) ":seconds"
	}
	comment {
	    set comments [db_nullify_empty_string $info_val]
	    set info_pairs(comments) ":comments"
	}
	mode {
	    set mode_num [db_nullify_empty_string [expr int($info_val)]]
	    set info_pairs(mode_num) ":mode_num"
	}
	copyright -
	vbr -
	stereo {
	    set "${info_key}_p" [db_nullify_empty_string \
				     [ad_decode $info_val 1 "t" 0 "f" ""]]
	    set "info_pairs(${info_key}_p)" ":${info_key}_p"
	}
	tracknum {
	    set tracknum [lindex [split $info_val /] 0]
	    set info_pairs(tracknum) ":tracknum"
	}
	layer -
 	version {
 	    set $info_key [db_nullify_empty_string [expr int($info_val)]]
 	    set info_pairs($info_key) ":$info_key"
 	}	    
	default {
	    set $info_key [db_nullify_empty_string $info_val]
	    set info_pairs($info_key) ":$info_key"
	}
    }
}    

# If no title just use the file_name
if {![info exists mp3_title]} {
    set mp3_title [mp3_name_from_file $client_filename]
}

if $update_entry_p {
    set mp3_id [db_string get_mp3_id "
select mp3_id 
from   mp3_mp3s 
where  file_path = :client_filename"]

    set extra_vars ""
    foreach i_key [array names info_pairs] {
	append extra_vars "\n       $i_key = $info_pairs($i_key),"
    }

    if [catch { 
	db_transaction {
	    db_dml undelete_mp3 "
update mp3_mp3s
set    title = :mp3_title,
       $extra_vars
       deleted_p = 'f'
where  mp3_id = :mp3_id
"

	    db_dml fix_creation "
update acs_objects
set    creation_user = :creation_user,
       creation_ip = :creation_ip,
       creation_date = sysdate,
       context_id = :package_id
where object_id = :mp3_id
"
	}
    } errmsg ] {

	set complaint " The database did not accept your input. 
	See details for the error message below\n\n\t<p><b>$errmsg</b>"
	ad_return_error "Database Error" "$complaint" 
	return -code return
	
    } 

} else {

    set extra_vars ""
    foreach i_key [array names info_pairs] {
	append extra_vars "\n      $i_key => $info_pairs($i_key),"
    }

    set mp3_id [db_null]

    if [catch { 
	    db_transaction {
		set mp3_id [db_exec_plsql add_mp3 ""]
		db_dml update_mp3_info ""
	    }
    } errmsg ] {

	set complaint " The database did not accept your input. 
	See details for the error message below\n\n\t<p><b>$errmsg</b>"
	ad_return_error "Database Error" "$complaint" 
	return -code return
	
    } 
}
	
ad_return_template
	

