ad_page_contract {
    Scan the MP3StagingLocation directory for new files
} {
}

# modified 12/18/2001 to fix bugs when attempting to scan files with spaces
# ivan@anilak.com

set user_id [ad_conn user_id]
set package_id [ad_conn package_id]
set creation_ip [ad_conn "peeraddr"]
set creation_user [ad_conn "user_id"]

set file_list [get_list_of_mp3s]
set page_html ""
if {[llength $file_list] == 0} {append page_html "No files found in Staging Location"}

foreach file_name $file_list {
    set staging_filename "[ad_parameter MP3StagingLocation]/$file_name"
    if {![regexp {(([^/]*)\.[mM][Pp]3)$} $file_name match client_filename mp3_name] } {
	append page_html "Broke on $file_name<br>\n"
	continue
    }

    if [db_0or1row check_file_name "
        select deleted_p 
        from   mp3_mp3s
        where  file_path = :client_filename"] {

	if { $deleted_p == "f" } {
	    append page_html "$file_name is already in the db. Continuing.<br>\n"
            file delete -force "$staging_filename"
	    continue
	} else {
	    set update_entry_p 1
	}
    } else {
	set update_entry_p 0
    }

    set target_filename "[ad_parameter MP3FileLocation]/$client_filename"
    file copy -force -- "$staging_filename" "$target_filename"
    ns_log notice "DELETE:$staging_filename"
    file delete -force -- "$staging_filename"
    

set mp3_info_list [exec "[acs_root_dir]/packages/mp3-jukebox/bin/mp3_file_scan" "$target_filename"]
# Workaround due to plpgsql (as of PG 7.1.3) being unable to handle
# default values in functions):
# The number of items in "mp3_info_list" may vary so we initialize
# all the possible vars to "nullvalue" in advance to know they're always
# being set.

set nullvalue [db_null]

set mp3_title $nullvalue
set artist $nullvalue
set bitrate $nullvalue
set album $nullvalue
set tracknum $nullvalue
set genre $nullvalue
set year $nullvalue
set layer $nullvalue
set copyright_p $nullvalue
set version $nullvalue
set mode_num $nullvalue
set stereo_p $nullvalue
set frequency $nullvalue
set minutes $nullvalue
set seconds $nullvalue
set comments $nullvalue
set vbr_p $nullvalue
set state $nullvalue

foreach info_item $mp3_info_list {
    set info_key [string tolower [lindex $info_item 0]]
    set info_val [lindex $info_item 1]
    switch -exact $info_key {
	title {
	    set mp3_title [db_nullify_empty_string $info_val]
	    # title is included in the standard sql, so no need to add to info_pairs
	}
	mm {
	    set minutes [db_nullify_empty_string $info_val]
	    set info_pairs(minutes) ":minutes"
	}
	ss {
	    set seconds [db_nullify_empty_string $info_val]
	    set info_pairs(seconds) ":seconds"
	}
	comment {
	    set comments [db_nullify_empty_string $info_val]
	    set info_pairs(comments) ":comments"
	}
	mode {
	    set mode_num [db_nullify_empty_string [expr int($info_val)]]
	    set info_pairs(mode_num) ":mode_num"
	}
	copyright -
	vbr -
	stereo {
	    set "${info_key}_p" [db_nullify_empty_string \
				     [ad_decode $info_val 1 "t" 0 "f" ""]]
	    set "info_pairs(${info_key}_p)" ":${info_key}_p"
	}
	tracknum {
	    set tracknum [lindex [split $info_val /] 0]
	    set info_pairs(tracknum) ":tracknum"
	}
	layer -
 	version {
 	    set $info_key [db_nullify_empty_string [expr int($info_val)]]
 	    set info_pairs($info_key) ":$info_key"
 	}	    
	default {
	    set $info_key [db_nullify_empty_string $info_val]
	    set info_pairs($info_key) ":$info_key"
	}
    }
}        

    # Is there a title defined?
    if { ![llength [array get info_pairs title]] } {
       set info_pairs(title) ":mp3_name"
    }


    if $update_entry_p {

	set mp3_id [db_string get_mp3_id "
select mp3_id 
from   mp3_mp3s 
where  file_path = :client_filename"]

	set extra_vars ""
	foreach i_key [array names info_pairs] {
	    append extra_vars ",\n       $i_key = $info_pairs($i_key)"
	}

	if [catch { 
	    db_transaction {
		db_dml undelete_mp3 "
update mp3_mp3s
set    deleted_p = 'f'
$extra_vars
where  mp3_id = :mp3_id
"
		db_dml fix_creation "
update acs_objects
set    creation_user = :creation_user,
       creation_ip = :creation_ip,
       creation_date = sysdate,
       context_id = :package_id
where object_id = :mp3_id
"
	    }
	} errmsg ] {

	    set complaint " The database did not accept your input. 
	See details for the error message below\n\n\t<p><b>$errmsg</b>"
	    ad_return_error "Database Error" "$complaint" 
	    return -code return
	
	} 

    } else {

        set extra_vars ""
	foreach i_key [array names info_pairs] {
		append extra_vars "\n      $i_key => $info_pairs($i_key),"
	}

	set mp3_id [db_null]

	if [catch { 
	    db_transaction {
		set mp3_id [db_exec_plsql add_mp3 ""]
		db_dml update_mp3_info ""
	    }
	} errmsg ] {


	    set complaint " The database did not accept your input. 
	See details for the error message below\n\n\t<p><b>$errmsg</b>"
	    ad_return_error "Database Error" "$complaint" 
	    return -code return
	}
    }

    unset info_pairs
    
    append page_html "<p><b>Completed $file_name.</b><p>\n"

}

# Stuck all this here so that title wouldn't collide in the namespace

set staging_location [ad_parameter MP3StagingLocation]
set page_title "Scanning MP3 Staging Location"
set title $page_title
set context [list $title]
